@@ -60,7 +60,7 @@ K_EXPORT_PLUGIN( AdBlockFactory( "adblock" ) )
 
 AdBlock::AdBlock(QObject *parent, const QVariantList & /*args*/) :
     Plugin(parent),
-    m_label(0), m_menu(0), m_elements(0)
+    m_menu(0), m_elements(0)
 {
     m_part = dynamic_cast<KHTMLPart *>(parent);
     if(!m_part)
@@ -104,12 +104,10 @@ AdBlock::~AdBlock()
 {
     KParts::StatusBarExtension *statusBarEx = KParts::StatusBarExtension::childObject(m_part);
 
-    if (statusBarEx)
-    {
-        statusBarEx->removeStatusBarItem(m_label);
-        delete m_label;
-    }
-    m_label = 0;
+    if (statusBarEx && m_label)
+        statusBarEx->removeStatusBarItem(m_label.data());
+    delete m_label.data();
+    m_label.clear();
     delete m_menu;
     m_menu = 0;
     delete m_elements;
@@ -127,18 +125,20 @@ void AdBlock::initLabel()
         return;
     }
 
-    m_label = new KUrlLabel(statusBarEx->statusBar());
+    KUrlLabel* label = new KUrlLabel(statusBarEx->statusBar());
 
     KIconLoader *loader = KIconLoader::global();
 
-    m_label->setFixedHeight(loader->currentSize(KIconLoader::Small));
-    m_label->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
-    m_label->setUseCursor(false);
-    m_label->setPixmap(loader->loadIcon("preferences-web-browser-adblock", KIconLoader::Small));
+    label->setFixedHeight(loader->currentSize(KIconLoader::Small));
+    label->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
+    label->setUseCursor(false);
+    label->setPixmap(loader->loadIcon("preferences-web-browser-adblock", KIconLoader::Small));
 
-    statusBarEx->addStatusBarItem(m_label, 0, false);
-    connect(m_label, SIGNAL(leftClickedUrl()), this, SLOT(slotConfigure()));
-    connect(m_label, SIGNAL(rightClickedUrl()), this, SLOT(contextMenu()));
+    statusBarEx->addStatusBarItem(label, 0, false);
+    connect(label, SIGNAL(leftClickedUrl()), this, SLOT(slotConfigure()));
+    connect(label, SIGNAL(rightClickedUrl()), this, SLOT(contextMenu()));
+    
+    m_label = label;
 }
 
 
@@ -279,18 +279,18 @@ void AdBlock::updateFilters()
     for ( it = m_elements->begin(); it != m_elements->end(); ++it )
     {
 	AdElement &element = (*it);
-        QString url = element.url();
-        if (settings->isAdFiltered(url))
-	{
-	    element.setBlocked(true);
-            // TODO: when this information is available from KHTML
-            //element.setBlockedBy(i18n("Blocked by %1",settings->adFilteredBy(url)));
-	}
-        else
+
+        bool isWhitelist;
+        QString filter = settings->adFilteredBy(element.url(), &isWhitelist);
+        if (!filter.isEmpty())
         {
-            // TODO: as above
-            //QString w = settings->adFilteredBy(url);
-            //if (!w.isEmpty()) element.setBlockedBy(i18n("Allowed by %1",w));
+            if (!isWhitelist)
+            {
+                element.setBlocked(true);
+                element.setBlockedBy(i18n("Blocked by %1",filter));
+            }
+            else
+                element.setBlockedBy(i18n("Allowed by %1",filter));
         }
     }
 }
@@ -300,7 +300,7 @@ void AdBlock::updateFilters()
 // ----------------------------------------------------------------------------
 
 AdElement::AdElement() :
-  m_url(0), m_category(0), m_type(0), m_blocked(false) {}
+  m_blocked(false) {}
 
 AdElement::AdElement(const QString &url, const QString &category,
 		     const QString &type, bool blocked, const DOM::Node&node) :
