@@ -257,7 +257,7 @@ void DOMTreeView::showRecursive(const DOM::Node &pNode, const DOM::Node &node, u
   }
 
   //kDebug(90180) << node.nodeName().string() << " [" << depth << "]";
-  addElement (node, cur_item, false);
+  cur_item = addElement (node, cur_item, false);
   m_listView->setItemExpanded(cur_item, depth < m_expansionDepth);
 
   if(node.handle()) {
@@ -290,13 +290,13 @@ void DOMTreeView::showRecursive(const DOM::Node &pNode, const DOM::Node &node, u
 	cur_item = new DOMListViewItem(node, parent_item, cur_item);
       }
       //kDebug(90180) << "</" << node.nodeName().string() << ">";
-      addElement(element, cur_item, true);
+      cur_item = addElement(element, cur_item, true);
 //       m_listView->setItemExpanded(cur_item, depth < m_expansionDepth);
     }
   }
 }
 
-void DOMTreeView::addElement ( const DOM::Node &node,  DOMListViewItem *cur_item, bool isLast)
+DOMListViewItem* DOMTreeView::addElement( const DOM::Node &node,  DOMListViewItem *cur_item, bool isLast)
 {
   assert(cur_item);
   cur_item->setClosing(isLast);
@@ -371,6 +371,7 @@ void DOMTreeView::addElement ( const DOM::Node &node,  DOMListViewItem *cur_item
   if (cur_item && m_bHighlightHTML && node.ownerDocument().isHTMLDocument()) {
     highlightHTML(cur_item, nodeName);
   }
+  return cur_item;
 }
 
 void DOMTreeView::highlightHTML(DOMListViewItem *cur_item, const QString &nodeName)
@@ -567,11 +568,8 @@ void DOMTreeView::searchRecursive(DOMListViewItem* cur_item, const QString& sear
     m_listView->setItemExpanded(cur_item, false);
   }
 
-  DOMListViewItem* child = static_cast<DOMListViewItem *>(cur_item->child(0));
-  while( child ) {
-    searchRecursive(child, searchText, caseSensitivity);
-    child = static_cast<DOMListViewItem *>(m_listView->itemBelow(child));
-  }
+  for (int cp = 0; cp < cur_item->childCount(); ++cp)
+    searchRecursive(static_cast<DOMListViewItem*>(cur_item->child(cp)), searchText, caseSensitivity);
 }
 
 #if 0
@@ -752,17 +750,15 @@ void DOMTreeView::adjustDepth()
 
 }
 
-void DOMTreeView::adjustDepthRecursively(QTreeWidgetItem *cur_item,  uint currDepth)
+void DOMTreeView::adjustDepthRecursively(QTreeWidgetItem *curItem,  uint currDepth)
 {
-  if (!(cur_item == 0)) {
-    while( cur_item ) {
-      m_listView->setItemExpanded(cur_item, m_expansionDepth > currDepth);
-      assert(m_listView->topLevelItemCount() == 1);
-      QTreeWidgetItem *topLevelItem = m_listView->topLevelItem(0);
-      adjustDepthRecursively(topLevelItem, currDepth+1);
-      cur_item = m_listView->itemBelow(cur_item);
-    }
-  }
+  if (!curItem)
+	return;
+
+  m_listView->setItemExpanded(curItem, m_expansionDepth > currDepth);
+
+  for (int cp = 0; cp < curItem->childCount(); ++cp)
+    adjustDepthRecursively(curItem->child(cp), currDepth + 1);
 }
 
 void DOMTreeView::setMessage(const QString &msg)
@@ -1210,7 +1206,7 @@ void DOMTreeView::initializeStyleSheetsFromDocument(const DOM::Document &doc)
     QStringList strList = QStringList(str);
     QTreeWidgetItem *topLevel = new QTreeWidgetItem(strList);
     styleSheetsTree->addTopLevelItem(topLevel);
-  
+
     // list the content
     DOM::CSSStyleSheet cssSheet(sheet);
     if (!cssSheet.isNull()) {
@@ -1230,7 +1226,7 @@ void DOMTreeView::initializeOptionsFromCData(const DOM::CharacterData &cdata)
   initializeDOMInfoFromCData(cdata);
   initializeCSSInfoFromCData(cdata);
 }
- 
+
 void DOMTreeView::initializeDOMInfoFromCData(const DOM::CharacterData &cdata)
 {
   contentEditor->setText(cdata.data().string());
