@@ -320,7 +320,7 @@ bool KImGalleryPlugin::createHtml(const KUrl& url, const QString& sourceDirName,
 
     if ( imageDir.exists() && file.open(QIODevice::WriteOnly) ) {
         QTextStream stream(&file);
-        stream.setEncoding(QTextStream::Locale);
+        stream.setCodec(QTextCodec::codecForLocale());
 
         createHead(stream);
         createBody(stream, sourceDirName, subDirList, imageDir, url, imageFormat); //ugly
@@ -390,7 +390,7 @@ void KImGalleryPlugin::loadCommentFile()
         kDebug(90170) << "File opened.";
 
         QTextStream* m_textStream = new QTextStream(&file);
-        m_textStream->setEncoding(QTextStream::Locale);
+        m_textStream->setCodec(QTextCodec::codecForLocale());
 
         delete m_commentMap;
         m_commentMap = new CommentMap;
@@ -437,11 +437,11 @@ bool KImGalleryPlugin::createThumb( const QString& imgName, const QString& sourc
     const QString pixPath = sourceDirName + QLatin1String("/") + imgName;
 
     if (m_copyFiles) {
-        KUrl srcURL = KUrl::fromPathOrUrl(pixPath);
+        KUrl srcURL = KUrl(pixPath);
         //kDebug(90170) << "srcURL: " << srcURL;
-        KUrl destURL = KUrl::fromPathOrUrl(imgGalleryDir + QLatin1String("/images/") + imgName);
+        KUrl destURL = KUrl(imgGalleryDir + QLatin1String("/images/") + imgName);
         //kDebug(90170) << "destURL: " << destURL;
-        KIO::NetAccess::copy(srcURL, destURL, static_cast<KParts::Part *>(parent())->widget());
+        KIO::NetAccess::file_copy(srcURL, destURL, static_cast<KParts::Part *>(parent())->widget());
     }
 
     const QString imgNameFormat = imgName + extension(imageFormat);
@@ -484,7 +484,25 @@ bool KImGalleryPlugin::createThumb( const QString& imgName, const QString& sourc
             img = scaleImg;
             if (m_configDlg->colorDepthSet() == true )
             {
-                const QImage depthImg(img.convertDepth(m_configDlg->getColorDepth()));
+                QImage::Format format;
+                switch (m_configDlg->getColorDepth())
+                {
+                case 1:
+                    format = QImage::Format_Mono;
+                    break;
+                case 8:
+                    format = QImage::Format_Indexed8;
+                    break;
+                case 16:
+                    format = QImage::Format_RGB16;
+                    break;
+                case 32:
+                default:
+                    format = QImage::Format_RGB32;
+                    break;
+                }
+
+                const QImage depthImg(img.convertToFormat(format));
                 img = depthImg;
             }
         }
