@@ -248,7 +248,7 @@ RelLinksPlugin::RelLinksPlugin(QObject *parent, const QVariantList &)
     disableAll();
 
     // When the rendering of the HTML is done, we update the site navigation bar
-    m_part = dynamic_cast<KHTMLPart *>(parent);
+    m_part = qobject_cast<KHTMLPart *>(parent);
     if (!m_part)
         return;
 
@@ -438,7 +438,7 @@ void RelLinksPlugin::updateToolbar() {
 
 void RelLinksPlugin::guessRelations()
 {
-	m_part = dynamic_cast<KHTMLPart *>(parent());
+	m_part = qobject_cast<KHTMLPart *>(parent());
 	if (!m_part || m_part->document().isNull() )
 		return;
 
@@ -496,7 +496,7 @@ void RelLinksPlugin::guessRelations()
 /** Menu links */
 void RelLinksPlugin::goToLink(const QString & rel, int id) {
     // have the KHTML part open it
-    KHTMLPart *part = dynamic_cast<KHTMLPart *>(parent());
+    KHTMLPart *part = qobject_cast<KHTMLPart *>(parent());
     if (!part)
         return;
 
