@@ -23,55 +23,53 @@
 #include "OpenSearchManager.h"
 #include "WebShortcutWidget.h"
 
-#include <unistd.h>
-#include <QLineEdit>
-#include <QApplication>
-#include <DOM/Element>
-#include <DOM/HTMLCollection>
-#include <DOM/HTMLDocument>
-#include <DOM/HTMLElement>
-#include <DOM/Node>
-#include <DOM/NodeList>
-#include <kaction.h>
-#include <KBuildSycocaProgressDialog>
-#include <KCompletionBox>
-#include <kconfig.h>
-#include <ksharedconfig.h>
-#include <kdebug.h>
-#include <kdesktopfile.h>
-#include <kgenericfactory.h>
-#include <kglobal.h>
-#include <khtml_part.h>
-#include <kiconloader.h>
-#include <klocale.h>
-#include <kmimetype.h>
-#include <kprotocolmanager.h>
-#include <kstandarddirs.h>
-#include <kurifilter.h>
-#include <kservice.h>
-#include <kparts/mainwindow.h>
-#include <kactioncollection.h>
+#include <KDE/KAction>
+#include <KDE/KBuildSycocaProgressDialog>
+#include <KDE/KCompletionBox>
+#include <KDE/KConfigGroup>
+#include <KDE/KSharedConfig>
+#include <KDE/KDebug>
+#include <KDE/KDesktopFile>
+#include <KDE/KGenericFactory>
+#include <KDE/KGlobal>
+#include <KDE/KIconLoader>
+#include <KDE/KLocale>
+#include <KDE/KActionCollection>
+#include <KDE/KRun>
+#include <KDE/KMainWindow>
+#include <KDE/KStandardDirs>
+#include <KParts/Part>
+#include <KParts/BrowserExtension>
+#include <KParts/TextExtension>
+#include <KParts/HtmlExtension>
 
-#include <qpainter.h>
-#include <qmenu.h>
-#include <qtimer.h>
-#include <qstyle.h>
-#include <QPixmap>
-#include <QMouseEvent>
+
+//#include <kparts/mainwindow.h>
+
+#include <QtGui/QLineEdit>
+#include <QtGui/QApplication>
 #include <QtCore/QTimer>
+#include <QtGui/QMenu>
+#include <QtGui/QStyle>
+#include <QtGui/QPixmap>
+#include <QtGui/QPainter>
+#include <QtGui/QMouseEvent>
 #include <QtDBus/QtDBus>
 
+
 K_PLUGIN_FACTORY(SearchBarPluginFactory, registerPlugin<SearchBarPlugin>();)
 K_EXPORT_PLUGIN(SearchBarPluginFactory("searchbarplugin"))
 
 SearchBarPlugin::SearchBarPlugin(QObject *parent,
                                  const QVariantList &) :
     KParts::Plugin(parent),
-    m_searchCombo(0),
+    m_part(0),
+    m_popupMenu(0),
+    m_addWSWidget(0),
     m_searchMode(UseSearchProvider),
     m_urlEnterLock(false),
-    m_process(0),
-    m_openSearchManager(new OpenSearchManager(this))
+    m_openSearchManager(new OpenSearchManager(this)),
+    m_reloadConfiguration(false)
 {
     m_searchCombo = new SearchBarCombo(0);
     m_searchCombo->lineEdit()->installEventFilter(this);
@@ -82,9 +80,6 @@ SearchBarPlugin::SearchBarPlugin(QObject *parent,
                                      "Enter a search term. Click on the icon to change search mode or provider.</p>"));
     connect(m_searchCombo, SIGNAL(suggestionEnabled(bool)), this, SLOT(enableSuggestion(bool)));
 
-    m_popupMenu = 0;
-    m_addWSWidget = 0;
-
     m_searchComboAction = actionCollection()->addAction("toolbar_search_bar");
     m_searchComboAction->setText(i18n("Search Bar"));
     m_searchComboAction->setDefaultWidget(m_searchCombo);
@@ -97,17 +92,22 @@ SearchBarPlugin::SearchBarPlugin(QObject *parent,
 
     configurationChanged();
 
+    m_timer = new QTimer(this);
+    m_timer->setSingleShot(true);
+    connect(m_timer, SIGNAL(timeout()), SLOT(requestSuggestion()));
+
     // parent is the KonqMainWindow and we want to listen to PartActivateEvent events.
     parent->installEventFilter(this);
 
-    connect(m_searchCombo->lineEdit(), SIGNAL(textEdited(const QString &)), SLOT(searchTextChanged(const QString &)));
+    connect(m_searchCombo->lineEdit(), SIGNAL(textEdited(const QString &)),
+            SLOT(searchTextChanged(const QString &)));
     connect(m_openSearchManager, SIGNAL(suggestionReceived(const QStringList &)),
             SLOT(addSearchSuggestion(const QStringList &)));
     connect(m_openSearchManager, SIGNAL(openSearchEngineAdded(const QString &, const QString &, const QString &)),
             SLOT(openSearchEngineAdded(const QString &, const QString &, const QString &)));
-    m_timer = new QTimer(this);
-    m_timer->setSingleShot(true);
-    connect(m_timer, SIGNAL(timeout()), SLOT(requestSuggestion()));
+    
+    QDBusConnection::sessionBus().connect(QString(), QString(), "org.kde.KUriFilterPlugin",
+                                          "configure", this, SLOT(reloadConfiguration()));
 }
 
 SearchBarPlugin::~SearchBarPlugin()
@@ -118,16 +118,37 @@ SearchBarPlugin::~SearchBarPlugin()
     config.writeEntry("SuggestionEnabled", m_suggestionEnabled);
 
     delete m_searchCombo;
-    m_searchCombo = 0L;
-    delete m_process;
-    m_process=0L;
+    m_searchCombo = 0;
 }
 
 bool SearchBarPlugin::eventFilter(QObject *o, QEvent *e)
 {
     if (qobject_cast<KMainWindow*>(o) && KParts::PartActivateEvent::test(e)) {
         KParts::PartActivateEvent* partEvent = static_cast<KParts::PartActivateEvent *>(e);
-        m_part = qobject_cast<KParts::ReadOnlyPart *>(partEvent->part());
+        KParts::ReadOnlyPart *part = qobject_cast<KParts::ReadOnlyPart *>(partEvent->part());
+        //kDebug() << "Embeded part changed to " << part;
+        if (part && (part != m_part)) {
+            m_part = part;
+
+            // Delete the popup menu so a new one can be created with the
+            // appropriate entries the next time it is shown...
+            // ######## TODO: This loses the opensearch entries for the old part!!!
+            if (m_popupMenu) {
+              delete m_popupMenu;
+              m_popupMenu = 0;
+              m_addSearchActions.clear(); // the actions had the menu as parent, so they're deleted now
+            }
+
+            // Change the search mode if it is set to FindInThisPage since
+            // that feature is currently KHTML specific. It is also completely
+            // redundant and unnecessary.
+            if (m_searchMode == FindInThisPage && enableFindInPage())
+              nextSearchEntry();
+
+            connect(m_part, SIGNAL(completed()), this, SLOT(HTMLDocLoaded()));
+            connect(m_part, SIGNAL(completed(bool)), this, SLOT(HTMLDocLoaded()));
+            connect(m_part, SIGNAL(started(KIO::Job *)), this, SLOT(HTMLLoadingStarted()));
+        }
         // Delay since when destroying tabs part 0 gets activated for a bit, before the proper part
         QTimer::singleShot(0, this, SLOT(updateComboVisibility()));
     }
@@ -151,19 +172,16 @@ void SearchBarPlugin::nextSearchEntry()
 {
     if (m_searchMode == FindInThisPage) {
         m_searchMode = UseSearchProvider;
-        if (!m_searchEngines.isEmpty()) {
+        if (m_searchEngines.isEmpty())
+            m_currentEngine = QLatin1String("google");
+        else
             m_currentEngine = m_searchEngines.first();
-        } else {
-            m_currentEngine = "google";
-        }
     } else {
-        int index = m_searchEngines.indexOf(m_currentEngine);
-        ++index;
-        if (index >= m_searchEngines.count()) {
+        const int index = m_searchEngines.indexOf(m_currentEngine) + 1;
+        if (index >= m_searchEngines.count())
             m_searchMode = FindInThisPage;
-        } else {
+        else
             m_currentEngine = m_searchEngines.at(index);
-        }
     }
     setIcon();
 }
@@ -172,51 +190,46 @@ void SearchBarPlugin::previousSearchEntry()
 {
     if (m_searchMode == FindInThisPage) {
         m_searchMode = UseSearchProvider;
-        if (!m_searchEngines.isEmpty()) {
-            m_currentEngine = m_searchEngines.last();
-        } else {
-            m_currentEngine = "google";
-        }
+        if (m_searchEngines.isEmpty())
+            m_currentEngine = QLatin1String("google");
+        else
+            m_currentEngine =  m_searchEngines.last();
     } else {
-        int index = m_searchEngines.indexOf(m_currentEngine);
-        if (index == 0) {
+        const int index = m_searchEngines.indexOf(m_currentEngine) - 1;
+        if (index <= 0)
             m_searchMode = FindInThisPage;
-        } else {
-            --index;
+        else
             m_currentEngine = m_searchEngines.at(index);
-        }
     }
     setIcon();
 }
 
+// Called when activating the combobox (Key_Return, or item in popup or in completionbox)
 void SearchBarPlugin::startSearch(const QString &search)
 {
-    if (m_urlEnterLock || search.isEmpty() || !m_part)
+    if (m_urlEnterLock || search.isEmpty() || !m_part) {
         return;
+    }
+    m_timer->stop();
+    m_lastSearch = search;
+
     if (m_searchMode == FindInThisPage) {
-        KHTMLPart *part = dynamic_cast<KHTMLPart*>(m_part);
-        if (part) {
-            part->findText(search, 0);
-            part->findTextNext();
-        }
+        KParts::TextExtension* textExt = KParts::TextExtension::childObject(m_part);
+        if (textExt)
+            textExt->findText(search, 0);
     } else if (m_searchMode == UseSearchProvider) {
         m_urlEnterLock = true;
-        KService::Ptr service;
+        const KUriFilterSearchProvider& provider = m_searchProviders.value(m_currentEngine);
         KUriFilterData data;
-        QStringList list;
-        list << "kurisearchfilter" << "kuriikwsfilter";
-        if ( !m_currentEngine.isEmpty() )
-            service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(m_currentEngine));
-        if (service) {
-            const QString searchProviderPrefix = service->property("Keys").toStringList().first() + m_delimiter;
-            data.setData(searchProviderPrefix + search);
-        }
-
-        if (!service || !KUriFilter::self()->filterUri(data, list)) {
-            data.setData(QLatin1String("google") + m_delimiter + search);
-            KUriFilter::self()->filterUri(data, list);
+        data.setData(provider.defaultKey() + m_delimiter + search);
+        //kDebug() << "Query:" << (provider.defaultKey() + m_delimiter + search);
+        if (!KUriFilter::self()->filterSearchUri(data, KUriFilter::WebShortcutFilter)) {          
+            kWarning() << "Failed to filter using web shortcut:" << provider.defaultKey();
+            return;
         }
 
+        
+        
         KParts::BrowserExtension * ext = KParts::BrowserExtension::childObject(m_part);
         if (QApplication::keyboardModifiers() & Qt::ControlModifier) {
             KParts::OpenUrlArguments arguments;
@@ -243,29 +256,13 @@ void SearchBarPlugin::setIcon()
     if (m_searchMode == FindInThisPage) {
         m_searchIcon = SmallIcon("edit-find");
     } else {
-        KService::Ptr service;
-        KUriFilterData data;
-        QStringList list;
-        list << "kurisearchfilter" << "kuriikwsfilter";
-        if (!m_currentEngine.isEmpty())
-            service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(m_currentEngine));
-        if (service) {
-            const QString searchProviderPrefix = service->property("Keys").toStringList().first() + m_delimiter;
-            data.setData(searchProviderPrefix + "some keyword");
-        }
-
-        if (service && KUriFilter::self()->filterUri(data, list)) {
-            QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForUrl(data.uri()) + ".png");
-            if (iconPath.isEmpty()) {
-                m_searchIcon = SmallIcon("unknown");
-            }
-            else {
-                m_searchIcon = QPixmap(iconPath);
-            }
-        }
-        else {
-            m_searchIcon = SmallIcon("google");
-        }
+        const QString engine = (m_currentEngine.isEmpty() ? m_searchEngines.first() : m_currentEngine);
+        //kDebug() << "Icon Name:" << m_searchProviders.value(engine).iconName();
+        const QString iconName = m_searchProviders.value(engine).iconName();
+        if (iconName.startsWith(QLatin1Char('/')))
+            m_searchIcon = QPixmap(iconName);
+        else
+            m_searchIcon = SmallIcon(iconName);
     }
 
     // Create a bit wider icon with arrow
@@ -285,49 +282,31 @@ void SearchBarPlugin::setIcon()
 
 void SearchBarPlugin::showSelectionMenu()
 {
+    // Update the configuration, if needed, before showing the menu items...
+    if (m_reloadConfiguration)
+        configurationChanged();
+    
     if (!m_popupMenu) {
-        KUriFilterData data;
-        QStringList list;
-        list << "kurisearchfilter" << "kuriikwsfilter";
-
         m_popupMenu = new QMenu(m_searchCombo);
         m_popupMenu->setObjectName("search selection menu");
-        m_popupMenu->addAction(KIcon("edit-find"), i18n("Find in This Page"), this, SLOT(useFindInThisPage()));
-        m_popupMenu->addSeparator();
 
-        int i =- 1;
-        for (QStringList::ConstIterator it = m_searchEngines.constBegin(); it != m_searchEngines.constEnd(); ++it) {
-            i++;
-            KService::Ptr service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(*it));
-            if (!service) {
-                continue;
-            }
-            const QString searchProviderPrefix = service->property("Keys").toStringList().first() + m_delimiter;
-            data.setData(searchProviderPrefix + "some keyword");
-
-            if (KUriFilter::self()->filterUri(data, list))
-            {
-                QIcon icon;
-                const QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForUrl(data.uri()) + ".png");
-                if (iconPath.isEmpty()) {
-                    icon = KIcon("unknown");
-                } else {
-                    icon = QPixmap(iconPath);
-                }
-                QAction* action = m_popupMenu->addAction(icon, service->name());
-                action->setData(qVariantFromValue(i));
-            }
+        if (enableFindInPage()) {
+            m_popupMenu->addAction(KIcon("edit-find"), i18n("Find in This Page"), this, SLOT(useFindInThisPage()));
+            m_popupMenu->addSeparator();
         }
 
-        m_popupMenu->addSeparator();
+        for (int i=0, count=m_searchEngines.count(); i != count; ++i) {
+            const KUriFilterSearchProvider& provider = m_searchProviders.value(m_searchEngines.at(i));
+            QAction* action = m_popupMenu->addAction(KIcon(provider.iconName()), provider.name());
+            action->setData(qVariantFromValue(i));
+        }
 
         m_popupMenu->addSeparator();
         m_popupMenu->addAction(KIcon("preferences-web-browser-shortcuts"), i18n("Select Search Engines..."),
                                this, SLOT(selectSearchEngines()));
         connect(m_popupMenu, SIGNAL(triggered(QAction *)), SLOT(menuActionTriggered(QAction *)));
-    }
-    else { // the popup menu is already deleted, don't try to delete each action in m_addSearchActions, otherwise kaboom
-        foreach (KAction *action, m_addSearchActions) {
+    } else {
+        Q_FOREACH (KAction *action, m_addSearchActions) {
             m_popupMenu->removeAction(action);
             delete action;
         }
@@ -340,9 +319,9 @@ void SearchBarPlugin::showSelectionMenu()
         before = actions[actions.size() - 2];
     }
 
-    foreach (QString title, m_openSearchDescs.keys()) {
+    Q_FOREACH (const QString &title, m_openSearchDescs.keys()) {
         KAction *addSearchAction = new KAction(m_popupMenu);
-        addSearchAction->setText(i18n("Add ") + title + "...");
+        addSearchAction->setText(i18n("Add %1...", title));
         m_addSearchActions.append(addSearchAction);
         addSearchAction->setData(QVariant::fromValue(title));
         m_popupMenu->insertAction(before, addSearchAction);
@@ -365,108 +344,86 @@ void SearchBarPlugin::menuActionTriggered(QAction *action)
         m_searchMode = UseSearchProvider;
         m_currentEngine = m_searchEngines.at(id);
         setIcon();
-        KService::Ptr service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(m_currentEngine));
         m_openSearchManager->setSearchProvider(m_currentEngine);
         m_searchCombo->lineEdit()->selectAll();
         return;
     }
 
-    QString openSearchTitle = action->data().toString();
+    const QString openSearchTitle = action->data().toString();
     if (!openSearchTitle.isEmpty()) {
-        KHTMLPart *part = dynamic_cast<KHTMLPart *>(m_part);
-        if (part) {
-            QString openSearchHref = m_openSearchDescs.value(openSearchTitle);
-            KUrl url;
-            if (QUrl(openSearchHref).isRelative()) {
-                KUrl docUrl = part->url();
-                QString host = docUrl.scheme() + "://" + docUrl.host();
-                if (docUrl.port() != -1) {
-                    host += ":" + QString::number(docUrl.port());
-                }
-                url = KUrl(docUrl, openSearchHref);
+        const QString openSearchHref = m_openSearchDescs.value(openSearchTitle);
+        KUrl url;
+        if (QUrl(openSearchHref).isRelative()) {
+            KUrl docUrl = m_part->url();
+            QString host = docUrl.scheme() + QLatin1String("://") + docUrl.host();
+            if (docUrl.port() != -1) {
+                host += QLatin1String(":") + QString::number(docUrl.port());
             }
-            else {
-                url = KUrl(openSearchHref);
-            }
-            kDebug() << "Adding open search Engine: " << openSearchTitle << " : " << openSearchHref;
-            m_openSearchManager->addOpenSearchEngine(url, openSearchTitle);
+            url = KUrl(docUrl, openSearchHref);
         }
+        else {
+            url = KUrl(openSearchHref);
+        }
+        //kDebug() << "Adding open search Engine: " << openSearchTitle << " : " << openSearchHref;
+        m_openSearchManager->addOpenSearchEngine(url, openSearchTitle);
     }
 }
 
 void SearchBarPlugin::selectSearchEngines()
 {
-    m_process = new KProcess;
-
-    *m_process << "kcmshell4" << "ebrowsing";
-
-    connect(m_process, SIGNAL(finished(int,QProcess::ExitStatus)), SLOT(searchEnginesSelected(int,QProcess::ExitStatus)));
-
-    m_process->start();
-    if(!m_process->waitForStarted())
-    {
-        kDebug(1202) << "Couldn't invoke kcmshell4";
-        delete m_process;
-        m_process = 0;
-    }
-}
-
-void SearchBarPlugin::searchEnginesSelected(int exitCode, QProcess::ExitStatus exitStatus)
-{
-    Q_UNUSED(exitCode);
-    if(exitStatus == QProcess::NormalExit) {
-        KConfigGroup config(KGlobal::config(), "SearchBar");
-        config.writeEntry("CurrentEngine", m_currentEngine);
-        config.sync();
-        configurationChanged();
-    }
-    m_process->deleteLater();
-    m_process = 0;
+    KRun::runCommand("kcmshell4 ebrowsing", (m_part ? m_part->widget() : 0));
 }
 
 void SearchBarPlugin::configurationChanged()
 {
-    KConfigGroup config(KSharedConfig::openConfig("kuriikwsfilterrc"), "General");
-    m_delimiter = config.readEntry("KeywordDelimiter", ":").at(0);
-    const QString engine = config.readEntry("DefaultSearchEngine", "google");
-
-    QStringList favoriteEngines;
-    favoriteEngines << "google" << "google_groups" << "google_news" << "webster" << "dmoz" << "wikipedia";
-    favoriteEngines = config.readEntry("FavoriteSearchEngines", favoriteEngines);
-
     delete m_popupMenu;
     m_popupMenu = 0;
-    m_searchEngines.clear();
-    m_searchEngines << engine;
-    for (QStringList::ConstIterator it = favoriteEngines.constBegin(); it != favoriteEngines.constEnd(); ++it)
-        if(*it!=engine)
-            m_searchEngines << *it;
     m_addSearchActions.clear();
+    m_searchEngines.clear();
+    m_searchProviders.clear();
 
-    config = KConfigGroup(KGlobal::config(), "SearchBar");
+    KUriFilterData data;
+    data.setSearchFilteringOptions(KUriFilterData::RetrievePreferredSearchProvidersOnly);
+    data.setAlternateDefaultSearchProvider(QLatin1String("google"));
+
+    if (KUriFilter::self()->filterSearchUri(data, KUriFilter::NormalTextFilter)) {
+        m_delimiter = data.searchTermSeparator();
+        Q_FOREACH(const QString& engine, data.preferredSearchProviders()) {
+            //kDebug() << "Found search provider:" << engine;
+            const KUriFilterSearchProvider& provider = data.queryForSearchProvider(engine);
+            m_searchProviders.insert(provider.desktopEntryName(), provider);
+            m_searchEngines << provider.desktopEntryName();
+        }
+    }
+
+    //kDebug() << "Found search engines:" << m_searchEngines;
+    KConfigGroup config = KConfigGroup(KGlobal::config(), "SearchBar");
     m_searchMode = (SearchModes) config.readEntry("Mode", (int) UseSearchProvider);
-    m_currentEngine = config.readEntry("CurrentEngine", engine);
-
+    m_currentEngine = config.readEntry("CurrentEngine", m_searchEngines.first());
     m_suggestionEnabled = config.readEntry("SuggestionEnabled", true);
+    
     m_searchCombo->setSuggestionEnabled(m_suggestionEnabled);
-
-    KService::Ptr service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(m_currentEngine));
-
     m_openSearchManager->setSearchProvider(m_currentEngine);
+    
+    m_reloadConfiguration = false;
     setIcon();
 }
 
+void SearchBarPlugin::reloadConfiguration()
+{
+    // NOTE: We do not directly connect the dbus signal to the configurationChanged
+    // slot because our slot my be called before the filter plugins, in which case we
+    // simply end up retrieving the same configuration information from the plugin.
+    m_reloadConfiguration = true;
+}
+
 void SearchBarPlugin::updateComboVisibility()
 {
-    bool isBrowser = m_part && dynamic_cast<KHTMLPart *>(m_part);
-
-    m_searchComboAction->setVisible(isBrowser && !m_searchComboAction->associatedWidgets().isEmpty());
-
+    // NOTE: We hide the search combobox if the embeded kpart is ReadWrite
+    // because web browsers by their very nature are ReadOnly kparts...
+    m_searchComboAction->setVisible((!qobject_cast<KParts::ReadWritePart *>(m_part) &&
+                                     !m_searchComboAction->associatedWidgets().isEmpty()));
     m_openSearchDescs.clear();
-    if (isBrowser) {
-        connect(m_part, SIGNAL(completed()), this, SLOT(HTMLDocLoaded()));
-        connect(m_part, SIGNAL(started(KIO::Job *)), this, SLOT(HTMLLoadingStarted()));
-    }
 }
 
 void SearchBarPlugin::focusSearchbar()
@@ -476,21 +433,27 @@ void SearchBarPlugin::focusSearchbar()
 
 void SearchBarPlugin::searchTextChanged(const QString &text)
 {
-    m_searchCombo->clearSuggestions();
-
-    if (m_suggestionEnabled && m_searchMode != FindInThisPage &&
-            m_openSearchManager->isSuggestionAvailable() && !text.isEmpty()) {
-        if (m_timer->isActive()) {
-            m_timer->stop();
-        }
-
-        // 400 ms delay before requesting for suggestions, so we don't flood the provider with suggestion request
-        m_timer->start(400);
+    // Don't do anything if the user just activated the search for this text
+    // Popping up suggestions again would just lead to an annoying popup (#231213)
+    if (m_lastSearch == text) {
+        return;
     }
+
+    // Don't do anything if the user is still pressing on the mouse button
+    if (qApp->mouseButtons()) {
+        return;
+    }
+
+    // 400 ms delay before requesting for suggestions, so we don't flood the provider with suggestion request
+    m_timer->start(400);
 }
 
 void SearchBarPlugin::requestSuggestion() {
-    if (!m_searchCombo->lineEdit()->text().isEmpty()) {
+    m_searchCombo->clearSuggestions();
+
+    if (m_suggestionEnabled && m_searchMode != FindInThisPage &&
+        m_openSearchManager->isSuggestionAvailable() &&
+        !m_searchCombo->lineEdit()->text().isEmpty()) {
         m_openSearchManager->requestSuggestion(m_searchCombo->lineEdit()->text());
     }
 }
@@ -502,43 +465,35 @@ void SearchBarPlugin::enableSuggestion(bool enable)
 
 void SearchBarPlugin::HTMLDocLoaded()
 {
-    KHTMLPart *khtmlPart = static_cast<KHTMLPart *>(m_part);
-
-    DOM::HTMLDocument htmlDoc = khtmlPart->htmlDocument();
-    DOM::NodeList headNL = htmlDoc.getElementsByTagName("head");
-    if (headNL.length() < 1) {
+    if (!m_part || m_part->url().host().isEmpty())
         return;
-    }
+    
+    // Testcase for this code: http://search.iwsearch.net
+    KParts::HtmlExtension* ext = KParts::HtmlExtension::childObject(m_part);
+    KParts::SelectorInterface* selectorInterface = qobject_cast<KParts::SelectorInterface*>(ext);
 
-    DOM::Node headNode = headNL.item(0);
-    if (headNode.nodeType() != 1) {
-        return;
-    }
-
-    DOM::HTMLElement headEl = (DOM::HTMLElement) headNode;
-    DOM::NodeList linkNL = headEl.getElementsByTagName("link");
-
-    for (unsigned int i = 0; i < linkNL.length(); i++) {
-        if (linkNL.item(i).nodeType() == 1) {
-            DOM::HTMLElement linkEl = (DOM::HTMLElement)linkNL.item(i);
-            if (linkEl.getAttribute("rel") == "search" && linkEl.getAttribute("type") == "application/opensearchdescription+xml") {
-                if (headEl.getAttribute("profile") != "http://a9.com/-/spec/opensearch/1.1/") {
-                    kWarning() << "Warning: there is no profile attribute or wrong profile attribute in <head>, as specified by open search specification 1.1";
-                }
-
-                QString title = linkEl.getAttribute("title").string();
-                QString href = linkEl.getAttribute("href").string();
-                m_openSearchDescs.insert(title, href);
-            }
+    if (selectorInterface) {
+        //if (headElelement.getAttribute("profile") != "http://a9.com/-/spec/opensearch/1.1/") {
+        //    kWarning() << "Warning: there is no profile attribute or wrong profile attribute in <head>, as specified by open search specification 1.1";
+        //}
+        const QString query (QLatin1String("head > link[rel=\"search\"][type=\"application/opensearchdescription+xml\"]"));
+        const QList<KParts::SelectorInterface::Element> linkNodes = selectorInterface->querySelectorAll(query, KParts::SelectorInterface::EntireContent);
+        //kDebug() << "Found" << linkNodes.length() << "links in" << m_part->url();
+        Q_FOREACH(const KParts::SelectorInterface::Element& link, linkNodes) {
+            const QString title = link.attribute("title");
+            const QString href = link.attribute("href");
+            //kDebug() << "Found opensearch" << title << href;
+            m_openSearchDescs.insert(title, href);
+            // TODO associate this with m_part; we can get descs from multiple tabs here...
         }
     }
 }
 
 void SearchBarPlugin::openSearchEngineAdded(const QString &name, const QString &searchUrl, const QString &fileName)
 {
-    kDebug() << "New Open Search Engine Added: " << name << ", searchUrl " << searchUrl;
+    //kDebug() << "New Open Search Engine Added: " << name << ", searchUrl " << searchUrl;
     QString path = KGlobal::mainComponent().dirs()->saveLocation("services", "searchproviders/");
-    
+
     KConfig _service(path + fileName + ".desktop", KConfig::SimpleConfig);
     KConfigGroup service(&_service, "Desktop Entry");
     service.writeEntry("Type", "Service");
@@ -568,15 +523,15 @@ void SearchBarPlugin::openSearchEngineAdded(const QString &name, const QString &
 
 void SearchBarPlugin::webShortcutSet(const QString &name, const QString &webShortcut, const QString &fileName)
 {
-    kDebug() << "Web shortcut for: " << name << "set to: " << webShortcut;
+    Q_UNUSED(name);
     QString path = KGlobal::mainComponent().dirs()->saveLocation("services", "searchproviders/");
     KConfig _service(path + fileName + ".desktop", KConfig::SimpleConfig);
     KConfigGroup service(&_service, "Desktop Entry");
     service.writeEntry("Keys", webShortcut);
-
-    // Update filters in running applications...
-    QDBusMessage msg = QDBusMessage::createSignal("/", "org.kde.KUriFilterPlugin", "configure");
-    QDBusConnection::sessionBus().send(msg);
+    _service.sync();
+    
+    // Update filters in running applications including ourselves...
+    QDBusConnection::sessionBus().send(QDBusMessage::createSignal("/", "org.kde.KUriFilterPlugin", "configure"));
 
     // If the providers changed, tell sycoca to rebuild its database...
     KBuildSycocaProgressDialog::rebuildKSycoca(m_searchCombo);
@@ -615,7 +570,7 @@ SearchBarCombo::SearchBarCombo(QWidget *parent) :
     connect(this, SIGNAL(aboutToShowContextMenu(QMenu*)), SLOT(addEnableMenuItem(QMenu*)));
 
     // use our own item delegate to display our fancy stuff :D
-    completionBox()->setItemDelegate(new SearchBarItemDelegate(this)); 
+    completionBox()->setItemDelegate(new SearchBarItemDelegate(this));
 }
 
 const QPixmap &SearchBarCombo::icon() const
@@ -660,7 +615,6 @@ void SearchBarCombo::mousePressEvent(QMouseEvent *e)
 
     if (e->x() > x0 + 2 && e->x() < lineEdit()->x()) {
         emit iconClicked();
-
         e->accept();
     } else {
         KHistoryComboBox::mousePressEvent(e);
@@ -684,7 +638,7 @@ void SearchBarCombo::setSuggestionItems(const QStringList &suggestions)
         QListWidgetItem *item = new QListWidgetItem(suggestions.at(0));
         item->setData(Qt::UserRole, "suggestion");
         completionBox()->insertItem(size + 1, item);
-	const int suggestionCount = suggestions.count();
+        const int suggestionCount = suggestions.count();
         for (int i = 1; i < suggestionCount; i++) {
             completionBox()->insertItem(size + 1 + i, suggestions.at(i));
         }
@@ -694,6 +648,10 @@ void SearchBarCombo::setSuggestionItems(const QStringList &suggestions)
 
 void SearchBarCombo::clearSuggestions()
 {
+    // Removing items can change the current item in completion box,
+    // which makes the lineEdit emit textEdited, and we would then
+    // re-enter this method, so block lineEdit signals.
+    const bool oldBlock = lineEdit()->blockSignals(true);
     int size = completionBox()->count();
     if (!m_suggestions.isEmpty() && size >= m_suggestions.count()) {
         for (int i = size - 1; i >= size - m_suggestions.size(); i--) {
@@ -701,6 +659,7 @@ void SearchBarCombo::clearSuggestions()
         }
     }
     m_suggestions.clear();
+    lineEdit()->blockSignals(oldBlock);
 }
 
 
@@ -762,4 +721,9 @@ void SearchBarItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem
     }
 }
 
+bool SearchBarPlugin::enableFindInPage() const
+{
+    return true;
+}
+
 #include "searchbar.moc"
