@@ -33,6 +33,7 @@
 #include <kglobal.h>
 #include <kconfig.h>
 #include <kio/job.h>
+#include <kio/scheduler.h>
 #include <kservice.h>
 #include <kcomponentdata.h>
 #include <kmenu.h>
@@ -67,13 +68,14 @@ UAChangerPlugin::UAChangerPlugin( QObject* parent,
   connect( m_pUAMenu->menu(), SIGNAL( aboutToShow() ),
            this, SLOT( slotAboutToShow() ) );
 
-  m_pUAMenu->setEnabled ( false );
-
-  if ( parent )
-  {
+  if (parent) {
       m_part = qobject_cast<KParts::ReadOnlyPart *>(parent );
-      connect( m_part, SIGNAL(started(KIO::Job*)), this,
-               SLOT(slotStarted(KIO::Job*)) );
+      connect(m_part, SIGNAL(started(KIO::Job*)), this,
+              SLOT(slotEnableMenu()) );
+      connect(m_part, SIGNAL(completed()), this,
+              SLOT(slotEnableMenu()));
+      connect(m_part, SIGNAL(completed(bool)), this,
+              SLOT(slotEnableMenu()));
   }
 }
 
@@ -91,8 +93,8 @@ void UAChangerPlugin::slotReloadDescriptions()
 
 void UAChangerPlugin::parseDescFiles()
 {
-  KService::List list = KServiceTypeTrader::self()->query("UserAgentStrings");
-  if ( list.count() == 0 )
+  const KService::List list = KServiceTypeTrader::self()->query("UserAgentStrings");
+  if (list.isEmpty())
     return;
 
   m_mapAlias.clear();
@@ -163,7 +165,7 @@ void UAChangerPlugin::parseDescFiles()
       if(!menuKey.isOther)
         alias = i18nc("%1 = browser version (e.g. 2.0)", "Version %1", UA_PTOS("X-KDE-UA-VERSION"));
       else
-        alias = i18nc("%1 = browser name, %2 = browser version (e.g. Firefox, 2.0)", 
+        alias = i18nc("%1 = browser name, %2 = browser version (e.g. Firefox, 2.0)",
                       "%1 %2", UA_PTOS("X-KDE-UA-NAME"), UA_PTOS("X-KDE-UA-VERSION"));
     }
     else
@@ -198,20 +200,17 @@ void UAChangerPlugin::parseDescFiles()
   }
 }
 
-void UAChangerPlugin::slotStarted( KIO::Job* )
+void UAChangerPlugin::slotEnableMenu()
 {
   m_currentURL = m_part->url();
 
   // This plugin works on local files, http[s], and webdav[s].
   QString proto = m_currentURL.protocol();
   if (m_currentURL.isLocalFile() ||
-      proto.startsWith("http") || proto.startsWith("webdav"))
-  {
+      proto.startsWith("http") || proto.startsWith("webdav")) {
     if (!m_pUAMenu->isEnabled())
       m_pUAMenu->setEnabled ( true );
-  }
-  else
-  {
+  } else {
     m_pUAMenu->setEnabled ( false );
   }
 }
@@ -224,53 +223,66 @@ void UAChangerPlugin::slotAboutToShow()
     parseDescFiles();
   }
 
-  if (!m_bSettingsLoaded)
+  if (!m_bSettingsLoaded) {
     loadSettings();
+  }
 
-  int count = 0;
-  m_pUAMenu->menu()->clear();
-  m_pUAMenu->menu()->addTitle(i18n("Identify As")); // imho title doesn't need colon..
+  if (m_pUAMenu->menu()->actions().isEmpty()) // need to create the actions
+  {
+      m_pUAMenu->menu()->addTitle(i18n("Identify As")); // imho title doesn't need colon..
+
+      m_defaultAction = new QAction(i18n("Default Identification"), this);
+      m_defaultAction->setCheckable(true);
+      connect(m_defaultAction, SIGNAL(triggered()), this, SLOT(slotDefault()));
+      m_pUAMenu->menu()->addAction(m_defaultAction);
+
+      m_pUAMenu->menu()->addSeparator();
+
+      m_actionGroup = new QActionGroup(m_pUAMenu->menu());
+      AliasConstIterator map = m_mapAlias.constBegin();
+      for( ; map != m_mapAlias.constEnd(); ++map )
+      {
+          QMenu* browserMenu = m_pUAMenu->menu()->addMenu( m_mapBrowser.value(map.key()) );
+          BrowserGroup::ConstIterator e = map.value().begin();
+          for( ; e != map.value().end(); ++e )
+          {
+              QAction* action = new QAction(m_lstAlias[*e], m_actionGroup);
+              action->setCheckable(true);
+              action->setData(*e);
+              browserMenu->addAction(action);
+          }
+      }
+      connect(m_actionGroup, SIGNAL(triggered(QAction*)), this, SLOT(slotItemSelected(QAction*)));
+
+      m_pUAMenu->menu()->addSeparator();
+
+      /* useless here, imho..
+         m_pUAMenu->menu()->insertItem( i18n("Reload Identifications"), this,
+         SLOT(slotReloadDescriptions()),
+         0, ++count );*/
+
+      m_applyEntireSiteAction = new QAction(i18n("Apply to Entire Site"), this);
+      m_applyEntireSiteAction->setCheckable(true);
+      connect(m_applyEntireSiteAction, SIGNAL(triggered()), this, SLOT(slotApplyToDomain()));
+      m_pUAMenu->menu()->addAction(i18n("Apply to Entire Site"));
+
+      m_pUAMenu->menu()->addAction( i18n("Configure..."), this,
+                                    SLOT(slotConfigure()));
+  }
+
+  // Reflect current settings in the actions
 
   QString host = m_currentURL.isLocalFile() ? QFL1("localhost") : m_currentURL.host();
   m_currentUserAgent = KProtocolManager::userAgentForHost(host);
   //kDebug(90130) << "User Agent: " << m_currentUserAgent;
+  m_defaultAction->setChecked(m_currentUserAgent == KProtocolManager::defaultUserAgent());
 
-  int id = m_pUAMenu->menu()->insertItem( i18n("Default Identification"), this,
-                                      SLOT(slotDefault()), 0, ++count );
-  if( m_currentUserAgent == KProtocolManager::defaultUserAgent() )
-     m_pUAMenu->menu()->setItemChecked(id, true);
-
-  m_pUAMenu->menu()->addSeparator();
-
-  AliasConstIterator map = m_mapAlias.constBegin();
-  for( ; map != m_mapAlias.constEnd(); ++map )
-  {
-    KMenu *browserMenu = new KMenu;
-    BrowserGroup::ConstIterator e = map.value().begin();
-    for( ; e != map.value().end(); ++e )
-    {
-       int id = browserMenu->insertItem( m_lstAlias[*e], this, SLOT(slotItemSelected(int)), 0, *e );
-       if (m_lstIdentity[(*e)] == m_currentUserAgent)
-         browserMenu->setItemChecked(id, true);
-    }
-    m_pUAMenu->menu()->insertItem( m_mapBrowser[map.key()], browserMenu );
+  m_applyEntireSiteAction->setChecked(m_bApplyToDomain);
+  Q_FOREACH(QAction* action, m_actionGroup->actions()) {
+      const int id = action->data().toInt();
+      action->setChecked(m_lstIdentity[id] == m_currentUserAgent);
   }
 
-  m_pUAMenu->menu()->addSeparator();
-
-  /* useless here, imho..
-  m_pUAMenu->menu()->insertItem( i18n("Reload Identifications"), this,
-                                      SLOT(slotReloadDescriptions()),
-                                      0, ++count );*/
-
-  m_pUAMenu->menu()->insertItem( i18n("Apply to Entire Site"), this,
-                                      SLOT(slotApplyToDomain()),
-                                      0, ++count );
-  m_pUAMenu->menu()->setItemChecked(count, m_bApplyToDomain);
-
-  m_pUAMenu->menu()->addAction( i18n("Configure..."), this,
-                                      SLOT(slotConfigure()));
-
 }
 
 void UAChangerPlugin::slotConfigure()
@@ -280,23 +292,21 @@ void UAChangerPlugin::slotConfigure()
     KRun::runCommand (service->exec (), m_part->widget());
 }
 
-void UAChangerPlugin::slotItemSelected( int id )
+void UAChangerPlugin::slotItemSelected(QAction* action)
 {
+  const int id = action->data().toInt();
   if (m_lstIdentity[id] == m_currentUserAgent) return;
 
-  QString host;
   m_currentUserAgent = m_lstIdentity[id];
-  host = m_currentURL.isLocalFile() ? QFL1("localhost") : filterHost( m_currentURL.host() );
+  QString host = m_currentURL.isLocalFile() ? QFL1("localhost") : filterHost( m_currentURL.host() );
 
   KConfigGroup grp = m_config->group( host.toLower() );
   grp.writeEntry( "UserAgent", m_currentUserAgent );
+  //kDebug(90130) << "Writing out UserAgent=" << m_currentUserAgent << "for host=" << host;
   grp.sync();
 
-  // Update the io-slaves...
-  updateIOSlaves ();
-
   // Reload the page with the new user-agent string
-  m_part->openUrl( m_currentURL );
+  reloadPage();
 }
 
 void UAChangerPlugin::slotDefault()
@@ -345,21 +355,19 @@ void UAChangerPlugin::slotDefault()
   // Reset some internal variables and inform the http io-slaves of the changes.
   m_currentUserAgent = KProtocolManager::defaultUserAgent();
 
-  // Update the http io-slaves.
-  updateIOSlaves();
-
   // Reload the page with the default user-agent
-  m_part->openUrl( m_currentURL );
+  reloadPage();
 }
 
-void UAChangerPlugin::updateIOSlaves ()
+void UAChangerPlugin::reloadPage()
 {
   // Inform running http(s) io-slaves about the change...
-  QDBusMessage message =
-          QDBusMessage::createSignal("/KIO/Scheduler", "org.kde.KIO.Scheduler", "reparseSlaveConfiguration");
-  message << QString();
-  if (!QDBusConnection::sessionBus().send(message))
-    kWarning() << "UAChangerPlugin::updateIOSlaves: Unable to update running application!" ;
+  KIO::Scheduler::emitReparseSlaveConfiguration();
+
+  KParts::OpenUrlArguments args = m_part->arguments();
+  args.setReload(true);
+  m_part->setArguments(args);
+  m_part->openUrl(m_currentURL);
 }
 
 QString UAChangerPlugin::filterHost(const QString &hostname)
