@@ -52,22 +52,23 @@
 
 #include <cassert>
 
-#include <qwidget.h>
-#include <qtextstream.h>
+#include <qtextcodec.h>
 #include <qtextdocument.h>
-#include <qtreewidget.h>
-#include <qtimer.h>
 
 #include <kdebug.h>
 #include <ktar.h>
+#include <kglobal.h>
 #include <kauthorized.h>
 #include <kcharsets.h>
-
-#include <kfiledialog.h>
+#include <kmimetype.h>
 #include <kmessagebox.h>
 #include <kstringhandler.h>
+#include <kstandardguiitem.h>
 
 #include <khtml_part.h>
+
+#include <kio/job.h>
+
 #include <dom/css_rule.h>
 #include <dom/css_stylesheet.h>
 #include <dom/css_value.h>
@@ -141,6 +142,16 @@ ArchiveDialog::RecurseData::RecurseData(KHTMLPart *_part, QTextStream *_textStre
     Q_ASSERT( !document.isNull() );
 }
 
+static KHTMLPart *isArchivablePart(KParts::ReadOnlyPart *part)
+{
+    KHTMLPart *cp = qobject_cast<KHTMLPart *>(part);
+    if (! cp)
+        return NULL;
+    DOM::HTMLDocument domdoc( cp->htmlDocument() );
+    if (domdoc.isNull())
+        return NULL;
+    return cp;
+}
 
 
 
@@ -148,9 +159,12 @@ ArchiveDialog::ArchiveDialog(QWidget *parent, const QString &filename, KHTMLPart
   : KDialog(parent), m_top(part), m_job(NULL), m_uniqId(2), m_tarBall(NULL), m_filename(filename), m_widget(NULL)
 {
     setCaption(i18n("Web Archiver"));
-    setButtons(KDialog::Ok | KDialog::Cancel | KDialog::User1 );
-    setDefaultButton(KDialog::Ok);
+    setButtons(KDialog::Ok | KDialog::Cancel);
+    setButtonGuiItem(KDialog::Ok, KStandardGuiItem::close());
     setModal(false);
+    enableButtonOk(false);
+    setDefaultButton(KDialog::NoDefault);
+
     m_widget = new ArchiveViewBase(this);
     {
         QTreeWidgetItem *twi = m_widget->progressView->headerItem();
@@ -158,7 +172,6 @@ ArchiveDialog::ArchiveDialog(QWidget *parent, const QString &filename, KHTMLPart
         twi->setText(1, i18n("Url"));
     }
     setMainWidget(m_widget);
-    connect(this, SIGNAL(finished()), SLOT(slotDialogFinished()));
 
     KUrl srcURL = part->url();
     m_widget->urlLabel->setText(QString("<a href=\"") + srcURL.url() + "\">" +
@@ -171,10 +184,6 @@ ArchiveDialog::ArchiveDialog(QWidget *parent, const QString &filename, KHTMLPart
     //else
     //   m_document = part->document().ownerDocument();
 
-    enableButton(Ok, false );
-    showButton(KDialog::User1, false);
-    setButtonGuiItem(KDialog::Ok, KStandardGuiItem::close());
-
     m_tarBall = new KTar(filename, "application/x-gzip");
     m_archiveTime = QDateTime::currentDateTime().toTime_t();
 }
@@ -409,13 +418,14 @@ void ArchiveDialog::saveWebpages() {
 
 void ArchiveDialog::finishedArchiving(bool tarerror) {
     if (tarerror) {
-        KMessageBox::error(this, i18n("I/O error occured while writing to web archive file %1", m_tarBall->fileName()));
+        KMessageBox::error(this, i18n("I/O error occurred while writing to web archive file %1.", m_tarBall->fileName()));
     }
     m_tarBall->close();
 
     m_widget->progressView->sortItems(0, Qt::AscendingOrder);
+    setDefaultButton(KDialog::Ok);
+    setEscapeButton(KDialog::Ok);
     enableButtonOk(true);
-    setEscapeButton(Ok);
     enableButtonCancel(false);
 }
 
@@ -457,7 +467,7 @@ static void filterFrameMappings(KHTMLPart *part, Id2Part &result) {
     FOR_ITER(ROPartList, childParts, child_it) {
         // TODO It is not clear from browsing the source code of KHTML if *child_it may be NULL
         Q_ASSERT(*child_it);
-        KHTMLPart *cp = qobject_cast<KHTMLPart *>(*child_it);
+        KHTMLPart *cp = isArchivablePart(*child_it);
         if (cp) {
             existing_frames.insert( FuncObj(cp), cp );
         }
@@ -474,8 +484,8 @@ static void filterFrameMappings(KHTMLPart *part, Id2Part &result) {
             it.value() = exists_it.value();
     }
     FOR_ITER_TEMPLATE(IdRemoveList, beRemoved, rem_it) {
-        result.erase( (*rem_it) );
         kDebug(90110) << "removing insecure(?) frame='" << (*rem_it).key();
+        result.erase( (*rem_it) );
     }
 }
 
@@ -610,7 +620,7 @@ void ArchiveDialog::obtainURLsLower(KHTMLPart *part, int level) {
     {   // Limit lifetime of @c childParts
         ROPartList childParts( part->frames() );
         FOR_ITER(ROPartList, childParts, child_it) {
-            KHTMLPart *cp = qobject_cast<KHTMLPart *>(*child_it);
+            KHTMLPart *cp = isArchivablePart(*child_it);
             if (cp) {
                 obtainURLsLower(cp, level+1);
             }
@@ -827,7 +837,7 @@ bool ArchiveDialog::saveFrame(KHTMLPart *part, int level) {
     {   // Limit lifetime of @c childParts
         ROPartList childParts( part->frames() );
         FOR_ITER(ROPartList, childParts, child_it) {
-            KHTMLPart *cp = qobject_cast<KHTMLPart *>(*child_it);
+            KHTMLPart *cp = isArchivablePart(*child_it);
             if (cp) {
                 error = saveFrame(cp, level+1);
                 if (error) {
